import sys
import traceback
from datetime import datetime

from colorifix.colorifix import Color, paint
from emoji import emojize


def standard_message(update, top_message, bot_name, bot_url):
    """Build a standard log message"""
    user_name = update.effective_chat.username or update.effective_chat.first_name
    user_id = update.effective_chat.id
    user_mention = f"[{user_name}](tg://user?id={user_id})"
    now = datetime.now()
    return emojize(
        f"{top_message}\n"
        f":robot_face: [{bot_name}]({bot_url})\n"
        f":calendar: {now:%d.%m.%Y}\n"
        f":eight_o’clock: {now:%H:%M}\n"
        f":bust_in_silhouette: {user_mention} (`{user_id}`)\n"
    )


def send_log(bot, channel, message, document=None):
    """Send a simple message to a channel"""
    if document:
        bot.send_document(
            channel,
            document,
            caption=emojize(message),
            parse_mode="Markdown",
        )
    else:
        bot.send_message(channel, emojize(message), parse_mode="Markdown")


def error_handler(
    update,
    context,
    channel,
    message,
    logger=None,
    exception=None,
    extra_info=None,
    log=True,
):
    """Log an error on terminal and a Telegram channel from a Telegram bot"""
    # error log on terminal
    user_data = (
        context.user_data.get(chat.id)
        if update and (chat := update.effective_chat)
        else "None"
    )
    trace = (exception and exception.__traceback__) or sys.exc_info()[2]
    traceback_error = "".join(traceback.format_tb(trace))
    traceback_msg = (
        f"\n{paint(user_data,229)}\n\n"
        f"{paint(traceback_error,Color.GRAY)}\n"
        f"> {paint(exception or context.error,Color.RED)}"
    )
    if logger:
        logger.warning(traceback_msg)
    else:
        print(traceback_msg)
    # send message to error channel
    if log:
        open("error.log", "w+").write(
            f"{user_data}\n\n"
            f"{extra_info or '...'}\n\n"
            f"{traceback_error}\n"
            f"> {exception or context.error}"
        )
        send_log(context.bot, channel, message, open("error.log", "r"))


def not_authorized_handler(update, context, bot_name, bot_url, channel):
    """Log an unauthorized access on a Telegram bot"""
    top_msg = ":face_with_symbols_on_mouth: *NO AUTH USER* :face_with_symbols_on_mouth:"
    message = standard_message(update, top_msg, bot_name, bot_url)
    send_log(context.bot, channel, message)



from googleapiclient.discovery import build
from requests import get

GOOGLE_URL = "https://google.it/search"
HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
    "(KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36",
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
    "Accept-Language": "en-US,en;q=0.9,it;q=0.8,la;q=0.7",
    "Accept-Encoding": "gzip, deflate",
    "DNT": "1",
    "Connection": "keep-alive",
    "Upgrade-Insecure-Requests": "1",
}


def google_search(query, api_key, cse_id, results=10, **kwargs):
    """Google search with API and CSE (official method)"""
    service = build("customsearch", "v1", developerKey=api_key)
    res = service.cse().list(q=query, cx=cse_id, num=results, **kwargs).execute()
    return res.get("items")


async def google_search_pirate_async(session, query):
    """Async Google search via scraping and fake header (pirate method)"""
    async with session.get(
        GOOGLE_URL, headers=HEADERS, params={"q": query}
    ) as response:
        google_content = await response.read()
    return google_content


def google_search_pirate(query):
    """Google search via scraping and fake header (pirate method)"""
    return get(GOOGLE_URL, headers=HEADERS, params={"q": query}).text
